%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hund et al model
% 1) Circulation 110:3168-317, 2004
% 2) revised in JMCC 2008
% dog ventricular cell
%   
%    t   time variable
%    V   membrane potential
%
% Randomly vary 7 parameters
% 7 params altered by Shannon et al to simulate HF
% GK1_;GKs_;Gto_;kNaCa;kappa_rel;Vup;Kleak

clear all
load hund_fixed_params

%Define colors to use for graphs in program
colors = repmat('krgbmc',1,1000) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Step 1:  Define all constants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global F R T RTF ;
% Physical constants
% F = 96.5 ;                   % Faraday constant, coulombs/mmol
% R = 8.314 ;                  % gas constant, J/K
F = 96485 ;                  % Faraday's constant, C/mol
R = 8314 ;                   % gas constant, mJ/K
T = 310 ;                 % absolute temperature, K 
RTF = R*T/F ;

global Acap Vmyo Vss VJSR VNSR ;
% Cell geometry constants
Acap = 1.534e-4 ;            % cm^2
Vmyo = 25.84e-6 ;            % uL
Vss = 0.76e-6 ;              % uL
VJSR = 0.182e-6 ;            % uL
VNSR = 2.098e-6 ;            % uL

global Ko Nao Clo Cao ;
% Fixed ionic concentrations
% Initial conditions of others listed below
Ko = 3.0 ;                  % uM
Nao = 140 ;               % uM
Clo = 100 ;
Cao = 1.8 ;                 % uM

% % New global variables that will be defined in 'hund_ode.m'
% 
global KmCaMK  
global GNa_ dV_CaMK_max daj_CaMK_max GNaL_ dGNaL_CaMK_max KmCaMK_Na
global GK1_ GKr_ GKs_ pKNa GKp_ Gto_ 
global PCa PCab gamma_Cao gamma_Cai dtauf_max dtaufCa_max
global Kmto2 GClb_ CTKCl_ CTNaCl_ PCl 
global KmNa_NaK KmK_NaK INaK_ 
global kNaCa ksat eta KmNai KmNao KmCai KmCao KmCa_allo
global IpCa_ KmpCa Vup Kmup dKmPLB_CaMK_max Kleak CaNSR_max dJleak_CaMK_max 
global tau_diff tau_transfer 
global TRPNtot KmTRPN CMDNtot KmCMDN CSQNtot KmCSQN BSRtot KmBSR BSLtot KmBSL 
global CaMK0 KmCaM alpha_CaMK beta_CaMK

GNa_ = 8.25   ;     % mS/cm^2
dV_CaMK_max = -3.25 ;
daj_CaMK_max = -0.18 ;

GNaL_= 6.5e-3 ;
dGNaL_CaMK_max = 0.0095 ;

KmCaMK_Na = 0.06 ;

PCa = 2.43e-4   ;      % mS/cm^2
PCab = 2.25084e-7   ;      % mS/cm^2
gamma_Cao = 0.341 ;
gamma_Cai = 1 ;
dtauf_max = 20 ;                               % ms
dtaufCa_max = 10 ;                    % ms

GK1_ = 0.5    ;     % mS/cm^2
GKr_ = 0.0138542 ;
GKs_ = 0.028975 ;
pKNa = 0.01833 ;                  % relative permeability of IKs, Na to K
GKp_ = 2.76e-3 ;
Gto_ = 0.79    ;   % mS/cm^2

Kmto2 = 0.1502 ;
GClb_ = 2.25e-4 ;
CTKCl_ = 1.77e-5 ;
CTNaCl_ = 2.46108e-5 ;
PCl = 4e-7 ;

KmNa_NaK = 10 ;             % Half-saturation concentration of NaK pump (mM)
KmK_NaK = 1.5 ;             % Half-saturation concentration of NaK pump (mM)
INaK_ = 0.95875 ;           % Max. current through Na-K pump (uA/uF)

kNaCa = 5.3 ;
ksat = 0.27 ;
eta = 0.35 ;
KmNai = 12.3 ; 
KmNao = 87.5 ;
KmCai = 0.00136 ; % not clear if really changed or a typo 
% KmCai = 0.0036 ; 
KmCao = 1.3 ;
KmCa_allo = 1.25e-4 ;

% In 2008 version, Km smaller (0.05) than in 2004 (0.15)
% However, Km = 0.06 for Na current, needs a different variable
KmCaMK = 0.05 ;

global kappa_rel betatau Krel_inf dKrelinf_max Krel_tau 
kappa_rel = 0.1 ;                      % uA/uF
betatau = 5.25 ;                       % mM
Krel_inf = 1 ;                         % mM 
dKrelinf_max = 0.35 ;                  % mM?
Krel_tau = 0.0123 ;                    % mM

IpCa_ = 0.0575 ; % Max. Ca current through sarcolemmal Ca pump (uA/uF)
KmpCa = 0.5e-3 ; % Half-saturation concentration of sarcolemmal Ca pump (mM)

Vup = 0.008375 ; 
Kmup = 9.2e-4 ; 
dKmPLB_CaMK_max = 1.7e-4 ;
% dJupmax = 0.75 ;

Kleak = 0.00628125 ;
CaNSR_max = 15.0 ;
dJleak_CaMK_max = 1.5 ;

tau_diff = 0.2 ;
tau_transfer = 120 ;

% % Buffers in cytosol
TRPNtot = 70e-3 ;    
KmTRPN = 0.5e-3 ;    

CMDNtot = 50e-3 ;
KmCMDN = 2.38e-3 ;    

% % Buffers in JSR
CSQNtot = 10 ;
KmCSQN = 0.8 ;

% % Buffers in subspace
BSRtot = 0.047 ; 
KmBSR = 8.7e-4 ; 

BSLtot = 1.124 ; 
KmBSL = 0.0087 ;

CaMK0 = 0.05 ;
KmCaM = 0.001 ;

alpha_CaMK = 6e-4    ;   % mS/cm^2
beta_CaMK = 3e-5 ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Step 1b:  Set up randomly varying parameter set  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

baselineparameters = [GK1_;GKs_;Gto_;kNaCa;kappa_rel;Vup;Kleak] ;
    
n_parameters = length(baselineparameters) ;
sigma = 0.1823 ;     % standard deviation of variation of parameters

% % % Random parameters and number of variations defined in data file
% % % hund_fixed_params

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Step 2:  Define simulation, stimulus, and recording parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

stiminterval = 1000 ;      % interval bewteen stimuli, ms
n_stimuli = 200 ;   
tend =  n_stimuli*stiminterval ;              % end of simulation, ms
stimdelay = 20 ;
stimdur = 1 ;
stim_amp = 80 ;

stim_starts = stimdelay + stiminterval*(0:n_stimuli-1)  ;
stim_ends = stim_starts + stimdur ;

simints = 3*n_stimuli ;
for i=1:n_stimuli
  intervals(3*i-2,:) = [stiminterval*(i-1),stim_starts(i)] ;
  intervals(3*i-1,:) = [stim_starts(i),stim_ends(i)] ;
  intervals(3*i,:) = [stim_ends(i),stiminterval*i] ;
end
intervals(end,:) = [stim_ends(end),tend] ;

Istim = zeros(simints,1) ;
stimindices = 3*(1:n_stimuli) - 1 ;
Istim(stimindices) = -stim_amp ;

% % % Only save last AP
numbertokeep = 1 ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Step 3:  Set new parameters and initial conditions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for ii=1:variations
  newparams = allparameters(:,ii) ;
  newparamscell = num2cell(newparams) ;
  [GK1_,GKs_,Gto_,kNaCa,kappa_rel,Vup,Kleak] = deal(newparamscell{:}) ;
     
%     V = -94.7 ;
%     Cai = 0.0822e-3 ;
%     Cass = 0.0822e-3 ;
%     CaJSR = 1.25 ;  %[mmol/L]
%     CaNSR = 1.25 ;  %[mmol/L]
%     Nai = 9.71 ;
%     Ki = 142.82 ;
%     Cli = 19.53 ;
%     m = 2.46e-4 ;
%     h = 0.99869 ;
%     j = 0.99887 ;
%     hL = 0.99869 ;
%     d = 1e-4 ;
%     dp = 1 ;
%     f = 0.983 ;
%     f2 = 0.983 ;
%     fCa = 0.942 ;
%     fCa2 = 0.942 ;
%     xKr = 0.229 ;
%     xs1 = 1e-4 ;
%     xs2 = 1e-4 ;
%     ato = 3.742e-5 ;
%     ito1 = 1 ;
%     ito2 = 1 ;
%     aa = 0 ;
%     Orel = 0 ;
%     CAMK_trap = 0.001 ;
% 
%     statevar_i = [V,Cai,Cass,CaNSR,CaJSR,Nai,Ki,Cli,m,h,j,hL, ...
%     d,f,fCa,fCa2,xKr,xs1,xs2,ato,ito1,ito2,aa,Orel,CAMK_trap] ;
% 
%     %% If variable ssbefore=TRUE, set all state variables to steady-state values
%     %% Let rest for 60 seconds without any stimuli applied 
%     if (ssbefore)
%         [post,posstatevars] = ode15s(@dydt_hund08,[0,300000],statevar_i,[],0) ;
%         statevar_i = posstatevars(end,:) ;  %State_var_i set to steady state values
%     end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Step 4: Numerically solve odes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

    load hundics

    if (n_stimuli > numbertokeep)
        for i=1:simints-3*numbertokeep
            %Use steady state values calculated for ss_before if it was
            %used. If not, use initial values set in program
            [post,posstatevars] = ode15s(@dydt_hund08,intervals(i,:),statevar_i,[],Istim(i)) ;
            statevar_i = posstatevars(end,:) ;  %Just keep parameter values for last timepoint
            t = post(end) ;
        end % for
        statevars = statevar_i ;
        for i=simints-3*numbertokeep+1:simints
            [post,posstatevars] = ode15s(@dydt_hund08,intervals(i,:),statevar_i,[],Istim(i)) ;
            t = [t;post(2:end)] ;
            statevars = [statevars;posstatevars(2:end,:)] ;  %Append newly calculated values that should be retained to the initial values
            statevar_i = posstatevars(end,:) ;
        end % for
    else    %If there are fewer stimuli than stimuli to keep, run program for all
        t = 0 ;
        statevars = statevar_i ;   
        tdicesdex = 1 ;
        for i=1:simints
            %state vars here are taken from ss_before or initial
            %conditions, since only stimuli run should be kept
            [post,posstatevars] = ode15s(@dydt_hund08,intervals(i,:),statevar_i,[],Istim(i)) ;
            t = [t;post(2:end)] ;
            tdices(tdicesdex) = length(t) ;
            tdicesdex = tdicesdex + 1 ;
            statevars = [statevars;posstatevars(2:end,:)] ;
            statevar_i = posstatevars(end,:) ;
        end % for
    end % if

    t = t - min(t) ;    %Since t started after many initial runs, need to reset t to 0 here.

    outputcell = num2cell(statevars,1) ;
    nvariables = length(outputcell) ;

    [V,Cai,Cass,CaNSR,CaJSR,Nai,Ki,Cli,m,h,j,hL, ...
      d,f,fCa,fCa2,xKr,xs1,xs2,ato,ito1,ito2,aa,Orel,CAMK_trap] = ...
      deal(outputcell{:}) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %% Step 5:  Calculate dependent variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  times{ii} = t ;
  aps{ii} = V ;
  catransients{ii} = Cai ; 
  
% 
% %Define resting potential, this is initial value used for stimuli time
%     %values to be generated. Used as dependent variable and needed to
%     %calculate APD at 50 and 90% repolarization for triangulation measure
  Vrest(ii) = V(1) ;
    Vderiv = diff(V)./diff(t) ;
  [dVdtmax(ii),dexmax] = max(Vderiv) ;
%   %% Time of maximum dV/dt, consider this beginning of action potential.
%   %% Also use as additional output - max upstroke velocity
  tinit = t(dexmax) ;   
% 
%   %% Determine peak voltage of action potential, for two reasons
%   %% 1) Because repolarization must, by definition, occur after this
%   %% 2) To compute 50%, 90%, etc., must have this value
  [peakV,peakdex] = max(V) ;
  Vpeak(ii) = peakV ;
  tpeak = t(peakdex) ;  
%   
%     
%   %Calculate repolarization time as time when voltage first decreases to
%   %less than 60, at a time later than the peak voltage. 
%   %%NOTE::: OUTLIERS!!!
  repoldex = find(t > tpeak & V < -60) ;
  repoltime = t(repoldex(1)) ;
  APD(ii) = repoltime - tinit ;
%   
%  %Define Calcium transient amplitude - compute difference btwn max and min Cai value
  Ca_trans(ii) = max(Cai)-min(Cai);
  CaSR_tot = VJSR*CaJSR.*(1 + CSQNtot./(CaJSR+KmCSQN)) + CaNSR*VNSR ;
  CaSR_diastole(ii) = CaSR_tot(end) ;
%     
%  %Define maximum Calcium value in SR. Paper measures Calcium leak and
%  %emphasizes the affect of CaMKII on this value, but it is not
%  %experimentally measurable. Instead, Calcium ions inside SR should get at
%  %leak effect.
%  %%NOTE: Need to convert so that both values are in microliters
%     %Ca and buffer concentrations are now in mmol/L; Volumes are in microL
%     %=> result in mmol
%  CaJSR_max = max(CaJSR) ; 
%  %Note: CaNSR_max is defined above as the maximum NSR concentration allowed, so need to name this 
%  %variable differently to distinguish. Actual maximum achieved is much
%  %lower than maximum allowed
%  CaNSR_high_val = max(CaNSR);
% %  Second term is forumula for how much buffer is bound to Ca2+ -> multiply
% %  by volume to get amount in mmol

[maxCa,dexmax] = max(Cai) ;
peak_time(ii) = t(dexmax) - stimdelay ;

% compute decay time as 60% decay
decayvalue = min(Cai) + (maxCa - min(Cai))/exp(1) ; 
belowdices = find(Cai < decayvalue) ;
decaydices = find(belowdices > dexmax) ;
decay_time(ii) = t(belowdices(decaydices(1))) - t(dexmax) ;

extrema = local_extrema_filt(t,V);  
maxmin(ii) = extrema(1) ;
minmax(ii) = extrema(2) ;
t_secmax = t(extrema(3)) ;

%  
% % %Use local extrema program to find difference between peak AP and first
% % %local minimum. 
%

% % %Note: t_secmax is to calculate triangulation after max_min because need maximum outputs from
% % %the local_extrema function. If there are two local maximums, make sure
% % %that 30% repolarization occurs after the second max.
%     
% 
%  %Calculate APD 50 and 90% repolarized. Triangulation measure is time
%   %between these two
    V30_exact = 0.7*(Vpeak(ii) - Vrest(ii)) + Vrest(ii) ;
    V90_exact = 0.1*(Vpeak(ii) - Vrest(ii)) + Vrest(ii) ;
%     %These values are the correct Voltage measure for 30 and 90%
%     %repolarization. Now need to identify which values are closest to this
%     %in the dataset. 
%     %Find first place in dataset where voltage goes below these cutoff
%     %values, and use this time in triangulation calculation
    V30dex = find(t > t_secmax & V < V30_exact) ;
    V30_time = t(V30dex(1));
    V90dex = find(t > t_secmax & V < V90_exact); 
    V90_time = t(V90dex(1));
    %Calculate triangulation
    Triang(ii) = V90_time-V30_time;

if (mod(ii,20) == 0)
  disp(['Completed ',int2str(ii),' iterations'])
end
if (mod(ii,100) == 0)
 save hund_fixed_hypokalemia allparameters variations n_parameters times aps catransients ...
   APD Vrest Vpeak Triang dVdtmax Ca_trans CaSR_diastole peak_time decay_time maxmin minmax   
end

end %(variations for loop)

    
 save hund_fixed_hypokalemia allparameters variations n_parameters times aps catransients ...
   APD Vrest Vpeak Triang dVdtmax Ca_trans CaSR_diastole peak_time decay_time maxmin minmax   
    

