%% This program performs the revere PLS regression with all 32 outputs
%% first and then rejects the 16 'least independent' outputs sequentially
%% before performing the reverse regression again with only the 'best' 16
%% outputs.

load tt_fixed_params
goodcount = goodcount 
%% Either load in data stored in .mat file varyparameters
%% Or re-run all simulations
colors = repmat('krgbmc',1,50) ;

for i=1:goodcount

  tnow = actionpotentials{2*i-1} ;
  Vnow = actionpotentials{2*i} ;
  Canow = catransients{2*i} ;
  Vrest(i) = Vnow(1) ;


  Vderiv = diff(Vnow)./diff(tnow) ;
  [dVdtmax,dexmax] = max(Vderiv) ;
  %% Time of maximum dV/dt, consider this beginning of action potential
  tinit = tnow(dexmax) ;
  upstrokes(i) = dVdtmax ;  

  %% Then determine peak voltage of action potential, for two reasons
  %% 1) Because repolarization must, by definition, occur after this
  %% 2) To compute 50%, 90%, etc., must have this value
  [peakV,peakdex] = max(Vnow) ;
  Vpeak(i) = peakV ;
  tpeak = tnow(peakdex) ;
  repoldex = find(tnow > tpeak & Vnow < -60) ;
  repoltime = tnow(repoldex(1)) ;
  APD(i) = repoltime - tinit ;

  
  Ca_diastole = Canow(1) ;
  [Ca_peak,dexCa] = max(Canow) ;
  DCa(i) = Ca_peak-Ca_diastole ;


end

Y = [log(APD)',-Vrest',Vpeak',upstrokes',log(DCa)'] ;

clear tnow Vnow Canow Vderiv2 pos dexmax tinit peakV peakdex tpeak 
clear repoldex repoltime Ca_diastole Ca_peak dexCa

load derivs_fixed

Y = [Y,log(Vmax_min)',log(Vmin_max)',log(tmin_max)'];

clear actionpotentials

load TNNP_alternans_fixed
Y=[Y,log(BCL_alt)];

clear actionpotentials

load threshold_fixed

Y=[Y,Ithresh];
clear actionpotentials

load restitution_fixed
Y=[Y, log(maxslopes)'];
clear actionpotentials

load ca_metrics
Y=[Y, time_peak', log(decay_time)'];

load tt_fixed_pause
for i=1:goodcount

  tnow = actionpotentials{2*i-1} ;
  Vnow = actionpotentials{2*i} ;
  Canow = catransients{2*i} ;
  Vrest(i) = Vnow(1) ;


  Vderiv = diff(Vnow)./diff(tnow) ;
  [dVdtmax,dexmax] = max(Vderiv) ;
  %% Time of maximum dV/dt, consider this beginning of action potential
  tinit = tnow(dexmax) ;
  upstrokes(i) = dVdtmax ;  

  %% Then determine peak voltage of action potential, for two reasons
  %% 1) Because repolarization must, by definition, occur after this
  %% 2) To compute 50%, 90%, etc., must have this value
  [peakV,peakdex] = max(Vnow) ;
  Vpeak(i) = peakV ;
  tpeak = tnow(peakdex) ;
  repoldex = find(tnow > tpeak & Vnow < -60) ;
  repoltime = tnow(repoldex(1)) ;
  APD_pause(i) = repoltime - tinit ;
  Ca_diastole = Canow(1) ;
  [Ca_peak,dexCa] = max(Canow) ;
  DCa_pause(i) = Ca_peak-Ca_diastole ;


end

Y=[Y, log(APD_pause)',log(DCa_pause)'];
clear tnow Vnow Canow Vderiv2 pos dexmax tinit peakV peakdex tpeak 
clear repoldex repoltime Ca_diastole Ca_peak dexCa

load derivs_fixed_pause
Y = [Y,log(Vmax_min_pause)',log(Vmin_max_pause)',log(tmin_max_pause)'];

APD_and_DCa_diff

Y=[Y, APD_diff',DCa_diff'];

load tt_fixed_hypokalemia
for i=1:goodcount

  tnow = actionpotentials{2*i-1} ;
  Vnow = actionpotentials{2*i} ;
  Canow = catransients{2*i} ;
  Vrest(i) = Vnow(1) ;


  Vderiv = diff(Vnow)./diff(tnow) ;
  [dVdtmax,dexmax] = max(Vderiv) ;
  %% Time of maximum dV/dt, consider this beginning of action potential
  tinit = tnow(dexmax) ;
  upstrokes(i) = dVdtmax ;  

  %% Then determine peak voltage of action potential, for two reasons
  %% 1) Because repolarization must, by definition, occur after this
  %% 2) To compute 50%, 90%, etc., must have this value
  [peakV,peakdex] = max(Vnow) ;
  Vpeak(i) = peakV ;
  tpeak = tnow(peakdex) ;
  repoldex = find(tnow > tpeak & Vnow < -60) ;
  repoltime = tnow(repoldex(1)) ;
  APDhypo(i) = repoltime - tinit ;
  Ca_diastole = Canow(1) ;
  [Ca_peak,dexCa] = max(Canow) ;
  DCahypo(i) = Ca_peak-Ca_diastole ;


end

Y=[Y, log(APDhypo)'];
clear tnow Vnow Canow Vderiv2 pos dexmax tinit peakV peakdex tpeak 
clear repoldex repoltime Ca_diastole Ca_peak dexCa

load tt_fixed_hyperkalemia
for i=1:goodcount

  tnow = actionpotentials{2*i-1} ;
  Vnow = actionpotentials{2*i} ;
  Canow = catransients{2*i} ;
  Vrest(i) = Vnow(1) ;


  Vderiv = diff(Vnow)./diff(tnow) ;
  [dVdtmax,dexmax] = max(Vderiv) ;
  %% Time of maximum dV/dt, consider this beginning of action potential
  tinit = tnow(dexmax) ;
  upstrokes(i) = dVdtmax ;  

  %% Then determine peak voltage of action potential, for two reasons
  %% 1) Because repolarization must, by definition, occur after this
  %% 2) To compute 50%, 90%, etc., must have this value
  [peakV,peakdex] = max(Vnow) ;
  Vpeak(i) = peakV ;
  tpeak = tnow(peakdex) ;
  repoldex = find(tnow > tpeak & Vnow < -60) ;
  repoltime = tnow(repoldex(1)) ;
  APDhyper(i) = repoltime - tinit ;
  Ca_diastole = Canow(1) ;
  [Ca_peak,dexCa] = max(Canow) ;
  DCahyper(i) = Ca_peak-Ca_diastole ;


end

Y=[Y, log(APDhyper)'];
clear tnow Vnow Canow Vderiv2 pos dexmax tinit peakV peakdex tpeak 
clear repoldex repoltime Ca_diastole Ca_peak dexCa

load derivs_fixed_hypo
Y=[Y, log(Vmax_min_hypo)',log(Vmin_max_hypo)',log(tmin_max_hypo)'];
clear actionpotentials

load derivs_fixed_hyper
Y=[Y, log(Vmax_min_hyper)',log(Vmin_max_hyper)',log(tmin_max_hyper)'];

clear actionpotentials

load tt_fixed_intracellular
Y=[Y,Ca', na', k'];
clear actionpotentials

load frequency_fixed



for i=1:variations
  tslow = actionpotentials{2*i-1,1}; 
  Vslow = actionpotentials{2*i,1}; 
  Vderiv = diff(Vslow)./diff(tslow) ;
  [dVdtmax,dexmax] = max(Vderiv) ;
  %% Time of maximum dV/dt, consider this beginning of action potential
  tinit = tslow(dexmax) ;
  [peakV,peakdex] = max(Vslow) ;
  tpeak = tslow(peakdex) ;
  repoldex = find(tslow > tpeak & Vslow < -60) ;
  repoltime = tslow(repoldex(1)) ;
  APDslow = repoltime - tinit ;

  tfast = actionpotentials{2*i-1,2}; 
  Vfast = actionpotentials{2*i,2}; 
  if (length(tfast))
    Vderiv = diff(Vfast)./diff(tfast) ;
    [dVdtmax,dexmax] = max(Vderiv) ;
    %% Time of maximum dV/dt, consider this beginning of action potential
    tinit = tfast(dexmax) ;
    [peakV,peakdex] = max(Vfast) ;
    tpeak = tfast(peakdex) ;
    repoldex = find(tfast > tpeak & Vfast < -60) ;
    repoltime = tfast(repoldex(1)) ;
    APDfast = repoltime - tinit ;
  else
    APDfast = 1e6 ;
  end
  adaptation(i) = APDslow - APDfast ; 
end

goodtrials = find(adaptation > 0) ;
Y = Y(goodtrials,:) ;
adaptation = adaptation(goodtrials) ;
APD = APD(goodtrials) ;
adaptation = adaptation./APD ;
Y = [Y,log(adaptation)'] ;

allparameters = allparameters(:,goodtrials) ;
variations = length(goodtrials) ;
X = log(allparameters)' ;

mu = mean(Y) ;
dev = std(Y) ;
Ytestlow = ( Y < ones(variations,1)*(mu - 3*dev) ) ;
keep = find(~any(Ytestlow,2)) ;
Y = Y(keep,:) ;
X = X(keep,:) ;
variations = length(keep);

Ytesthigh = ( Y > ones(variations,1)*(mu + 3*dev) ) ;
keep = find(~any(Ytesthigh,2)) ;
Y = Y(keep,:) ;
X = X(keep,:) ;
variations = length(keep);

placeholder = X ;
X = Y ;
Y = placeholder ;
clear placeholder

nfact2keep = rank(X) ;
[T,P,W,Wstar,U,b,C,Bpls,Bpls_star,Xhat,Yhat,R2X,R2Y]=...
    PLS_nipals(X,Y,nfact2keep);


Glabels = { ...
  'G_N_a',...
  'G_N_a_b',...
  'G_C_a',...
  'G_C_a_b',...
  'G_t_o', ...
  'G_K_r',...
  'G_K_s',...
  'G_K_1',...
  'G_p_K',...
  'K_N_a_K',...
  'K_N_C_X', ...
  'K_r_e_l_1',...
  'K_r_e_l_2',...
  'K_l_e_a_k',...
  'K_u_p',...
  'K_p_C_a'...
  };



figure
handle = gcf ;
set(handle,'Position',[20,20,1100,950])
set(handle,'PaperPosition',[1 1 11 9.5]) ;
figure(11)
for i=1:4
    subplot(2,2,i)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end
figure(12)

for i=5:8
    subplot(2,2,i-4)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end
figure(13)

for i=9:12
    subplot(2,2,i-8)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end
figure(14)
for i=13:16
    subplot(2,2,i-12)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end



SSYT=sum((Y-ones(variations,1)*mean(Y)).^2);
SSYR=sum((Yhat-ones(variations,1)*mean(Y)).^2);
R2each=SSYR./SSYT

[samples,outputs] = size(X) ;
[samples,conductances] = size(Y) ;

outputlist = { ...
'APD', ...
'Vrest', ...
'Vpeak', ...
'dVdtmax', ...
'DCa',...
'Vmaxmin', ...
'Vminmax', ...
'tminmax', ...
'alternans threshold', ...
'stimulation threshold', ...
'restitution slope', ...
'time to peak',...
'decay time',...
'APD_pause',...
'DCa_pause',...
'Vmaxmin_pause',...
'Vminmax_pause',...
'tminmax_pause',...
'APD_diff',...
'DCa_diff',...
'APD_hypo',...
'APD_hyper',...
'Vmaxmin_hypo',...
'Vminmax_hypo',...
'tminmax_hypo',...
'Vmaxmin_hyper',...
'Vminmax_hyper',...
'tminmax_hyper',...
'intracellular Ca',...
'intracellular Na',...
'intracellular K',...
'frequency adaptation',...
} ;

oldr2 = sum(R2Y) ;
for iiii=outputs:-1:conductances+1
  rsquared = zeros(iiii,1) ;
  for iii=1:iiii
    Xtest = X(:,[1:iii-1,iii+1:end]) ;
    nfact2keep = rank(Xtest) ;
    [T,P,W,Wstar,U,b,C,Bpls,Bpls_star,Xhat,Yhat,R2X,R2Y]=...
      PLS_nipals(Xtest,Y,nfact2keep);
    rsquared(iii) = sum(R2Y) ;  
  end
  [newr2,reject] = max(rsquared) ;
  disp(['Now remove the ',int2str(reject),'th column']) ;
  disp(['This corresponds to ',outputlist{reject}]) ;
  disp(['R^2 decreases from ',num2str(oldr2),' to ',num2str(newr2)]) ;
  X = X(:,[1:reject-1,reject+1:end]) ;
  outputlist = outputlist([1:reject-1,reject+1:end]) ;
  oldr2 = newr2 ;
end


nfact2keep = rank(X) ;
[T,P,W,Wstar,U,b,C,Bpls,Bpls_star,Xhat,Yhat,R2X,R2Y]=...
    PLS_nipals(X,Y,nfact2keep);

% figure
% handle = gcf ;
% set(handle,'Position',[20,20,1100,950])
% set(handle,'PaperPosition',[1 1 11 9.5]) ;
% for i=1:16
%     subplot(8,2,i)
%     hold on
%     plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
%     plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
%     plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
%     xlabel(['True G',Glabels{i}])
%     ylabel(['Predicted G',Glabels{i}])
%     axis([plotmin plotmax plotmin plotmax])
%     plot([plotmin,plotmax],[plotmin,plotmax],'k:')
% end

figure
handle = gcf ;
set(handle,'Position',[20,20,1100,950])
set(handle,'PaperPosition',[1 1 11 9.5]) ;

figure(20)
for i=1:4
    subplot(2,2,i)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end

figure(21)
for i=5:8
    subplot(2,2,i-4)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end
figure(22)
for i=9:12
    subplot(2,2,i-8)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end
figure(23)
for i=13:16
    subplot(2,2,i-12)
    hold on
    plot(exp(Y(:,i)),exp(Yhat(:,i)),'ro')
    plotmin = min(min([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    plotmax = max(max([exp(Y(:,i)),exp(Yhat(:,i))])) ;
    xlabel(['True G',Glabels{i}])
    ylabel(['Predicted G',Glabels{i}])
    axis([plotmin plotmax plotmin plotmax])
    plot([plotmin,plotmax],[plotmin,plotmax],'k:')
end


SSYT=sum((Y-ones(variations,1)*mean(Y)).^2);
SSYR=sum((Yhat-ones(variations,1)*mean(Y)).^2);
R2each=SSYR./SSYT


