% Ten Tusscher et al model
% Am J Physiol 286: H1573H1589, 2004.
% human ventricular cell 
% improved version of Priebe-Beucklemann
%    
%    t   time variable
%    V   membrane potantial
load tt_fixed_params
colors = repmat('krgbmc',1,500) ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 1:  Define all constants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global F R T RTF ;
% Physical constants
F = 96.4867 ;                % Faraday constant, coulombs/mmol
R = 8.314 ;                  % gas constant, J/(K mol)
T = 273+37 ;                 % absolute temperature, K 
RTF = R*T/F ;

global Cm S ;
Cm = 1 ;                      % uF/cm2
S = 0.2 ;                     % surface area to volume ratio

global Vmyo VSR ;
Vmyo = 16.404 ;               % pL
VSR = 1.094 ;                 % pL

global Ko Nao Cao ;
Ko = 2000 ;                   % uM
Nao = 140000 ;                % uM
Cao = 2000 ;                  % uM

global GNa_ PCa_ GK1_ GKr_ GKs_ Gto_ ;
global GpK_ GpCa_ GNab_ GCab_ pKNa ;
GNa_ = 14.838 ;               % nS/pF
PCa_ = 1.75e-4 ;              % cm3 uF-1 s-1 
GK1_ = 5.405 ;                % nS/pF
GKr_ = 0.096 ;                % nS/pF

GpK_ = 0.0146 ;               % nS/pF
GpCa_ = 0.025 ;               % nS/pF
GNab_ = 2.9e-4 ;              % nS/pF
GCab_ = 5.92e-4 ;             % nS/pF

% % % Epicardial cell
Gto_ = 0.294 ;                % nS/pF
GKs_ = 0.245 ;                % nS/pF

% % % % M cell
% Gto_ = 0.294 ;                % nS/pF
% GKs_ = 0.062 ;                % nS/pF
% 
% % % % Endocardial cell
% Gto_ = 0.073 ;                % nS/pF
% GKs_ = 0.245 ;                % nS/pF
% 

% % % % Epicardial cell, M cell
% Gto_ = 0.294 ;                 % nS/pF
% % % % Endocardial cell
% Gto_ = 0.073 ;                 % nS/pF
% 
% % % % Epicardial cell, endocardial cell
% GKs_ = 0.245 ;                 % nS/pF
% % % % M cell
% GKs_ = 0.062 ;                 % nS/pF

pKNa = 0.03 ;                  % relative permeability, Na to K

global INaK_ kNaCa Iup_ Vleak
% Maximum rates of intracellular transport mechanisms
INaK_ = 1.362 ;                % pA/pF
kNaCa = 1000 ;                 % pA/pF
Iup_ = 0.425 ;                 % uM/ms
Vleak = 8e-5 ;                 % ms-1

global KmNa KmCa ksat alpha_ncx eta KmNai KmKo KpCa Kmup ;
% global eta ksat KmNa KmCa KmfCa KmK1 KmNai KmKo KmpCa Kmup  
% global KmNa KmCa KmNai KmKo sigma ;
% % values needed for calculation of NCX current
% Several saturation constants, nomenclature gets tricky
% NCX
KmNa = 87500 ;                 % uM
KmCa = 1380 ;                  % uM
ksat = 0.1 ;                   % unitless
alpha_ncx = 2.5 ;              % unitless
eta = 0.35 ;                   % unitless, actually gamma in paper
% Na-K pump
KmNai = 40000 ;                % uM
KmKo = 1000 ;                  % uM
% Sarcolemmal Ca pump
KpCa = 0.5 ;                    % uM
% SERCA
Kmup = 0.25 ;                  % uM

global arel brel crel ;
arel = 16.464 ;                 % uM/ms
brel = 250 ;                   % uM
crel = 8.232 ;                  % uM/ms

% % % Buffering parameters
% % % Considered generic cytotolic and SR buffers
global Bufc Kbufc BufSR KbufSR 
Bufc = 150 ;                % uM
Kbufc = 1 ;                   % uM
BufSR = 10000 ;              % uM
KbufSR = 300 ;                 % uM 

baselineparameters = [GNa_;GNab_;PCa_;GCab_;Gto_;GKr_;GKs_;GK1_;GpK_; ...
  INaK_;kNaCa;arel;crel;Vleak;Iup_;GpCa_] ;

n_parameters = length(baselineparameters) ;
variations = 300 ;

sigma = 0.1 ;     % standard deviation of variation of parameters
% sigma = 0 ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Step 2:  Define simulation, stimulus, and recording parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

stiminterval = 1000 ;      % interval bewteen stimuli, ms
stimdelay = 50 ;
stimdur = 2 ;
stim_amp = 30 ;


tend = 20*stiminterval ;              % end of simulation, ms

n_stimuli = ceil((tend-stimdelay)/stiminterval) ;
stim_starts = stimdelay + stiminterval*(0:n_stimuli-1)  ;
stim_ends = stim_starts + stimdur ;

simints = 3*n_stimuli ;
for i=1:n_stimuli
  intervals(3*i-2,:) = [stiminterval*(i-1),stim_starts(i)] ;
  intervals(3*i-1,:) = [stim_starts(i),stim_ends(i)] ;
  intervals(3*i,:) = [stim_ends(i),stiminterval*i] ;
end
intervals(end,:) = [stim_ends(end),tend] ;

Istim = zeros(simints,1) ;
stimindices = 3*(1:n_stimuli) - 1 ;
Istim(stimindices) = -stim_amp ;

% % % Previously, model saved all action potentials
% % % Now, only save last two, but can vary this if need be
numbertokeep = 1 ;

figure(1)
clf
hold on

goodcount = 1 ;
hcgcount = 1 ;

for ii=1:variations
    
  newparams = allparameters(:,ii) ;
  newparamscell = num2cell(newparams) ;
  [GNa_,GNab_,PCa_,GCab_,Gto_,GKr_,GKs_,GK1_,GpK_, ...
    INaK_,kNaCa,arel,crel,Vleak,Iup_,GpCa_] = deal(newparamscell{:}) ;
    
V = -85 ;
m = 0 ;
h = 1 ;
j = 1 ;
d = 0 ;
f = 1 ;
fCa = 1 ;
r = 0 ;
s = 1 ;
xs = 0 ;
xr1 = 1 ;
xr2 = 1 ;
g = 0 ;
Cai = 0.1 ;
CaSR = 1000 ;
Nai = 10000 ;
Ki = 130000 ;

statevar_i = [V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] ;

[post,posstatevars] = ode15s(@dydt_tentusscher,[0,30000],statevar_i,[],0) ;
statevar_i = posstatevars(end,:) ;

  if (n_stimuli > numbertokeep)
    for i=1:simints-3*numbertokeep
      [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;
      statevar_i = posstatevars(end,:) ;
      t = post(end) ;
    end % for
    statevars = statevar_i ;
    for i=simints-3*numbertokeep+1:simints
      [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ;
      t = [t;post(2:end)] ;
      statevars = [statevars;posstatevars(2:end,:)] ;
      statevar_i = posstatevars(end,:) ;
    end % for
  else
    t = 0 ;
    statevars = statevar_i ;
    for i=1:simints
       [post,posstatevars] = ode15s(@dydt_tentusscher,intervals(i,:),statevar_i,[],Istim(i)) ; 
      statevars = [statevars;posstatevars(2:end,:)] ;
      statevar_i = posstatevars(end,:) ;
    end % for
  end % if

t = t - min(t) ;

outputcell = num2cell(statevars,1) ;
nvariables = length(outputcell) ;

[V,m,h,j,d,f,fCa,r,s,xs,xr1,xr2,g,Cai,CaSR,Nai,Ki] = deal(outputcell{:}) ;

figure(1)
hold on
plot(t,V,colors(ii))


if (V(end) > -70)
  hcgaps{2*hcgcount-1} = t ;
  hcgaps{2*hcgcount} = V ;
  hcgcai{2*hcgcount-1} = t ;
  hcgcai{2*hcgcount} = Cai ;
  hcgparams(:,hcgcount) = newparams ;
  hcgcount = hcgcount + 1 ;
else
  actionpotentials{2*goodcount-1} = t ;
  actionpotentials{2*goodcount} = V ;
  catransients{2*goodcount-1} = t ;
  catransients{2*goodcount} = Cai ;
  allparameters(:,goodcount) = newparams ;
  goodcount = goodcount + 1 ;
end

if (mod(ii,50) == 0)
  disp(['Completed ',int2str(ii),' iterations'])
end

end

goodcount = goodcount - 1 ;
hcgcount = hcgcount - 1 ;

save tt_fixed_hypokalemia allparameters actionpotentials catransients goodcount n_parameters stiminterval sigma 
if (hcgcount)
  save TThcg_1 hcgparams hcgaps hcgcai hcgcount n_parameters 
end

